-- Design: Top_Level
-- Functionality: Top level wrapper fir for Fusion Design Hand Book CoreABC example. 
-- Version: 7.3
-- Last Update: 05/21/2007
library ieee;
use ieee.std_logic_1164.all;

entity TOP_LEVEL is port (
  CLK :  in std_logic;
  RESETn: in std_logic;
  AC1 : in std_logic;
  AT2 : in std_logic;
  ATRETURN : in std_logic;
  AV0 : in std_logic;
  AV1 : in std_logic;
  AG3 : out std_logic;
  AG4 : out std_logic;
  Flags : out std_logic_vector(0 downto 0);
  VAREF : inout std_logic
  );
end TOP_LEVEL;

architecture arch of TOP_LEVEL is 

-- CoreABC + CoreAI system created in CoreConsole
component CoreABC_AI_Test 
    -- Port list
    port(
        -- Inputs
        AC1 : in std_logic;
        AT2 : in std_logic;
        ATRETURN : in std_logic;
        AV0 : in std_logic;
        AV1 : in std_logic;
        INITADDR : in std_logic_vector(11 downto 0);
        INITDATA : in std_logic_vector(8 downto 0);
        INITDATVAL : in std_logic;
        INITDONE : in std_logic;
        NSYSRESET : in std_logic;
        SYSCLK : in std_logic;
        -- Outputs
        AG3 : out std_logic;
        AG4 : out std_logic;
        Flags : out std_logic_vector(0 downto 0);
        -- Inouts
        VAREF : inout std_logic
    );
end component;

-- PLL block generates two clock networks. A 40MHz clock used as system clock and and a 10MHz
-- clock used for initialzation 
component PLL_Block 
    port(POWERDOWN, CLKA : in std_logic;  LOCK, GLA, GLC : out 
        std_logic;  OADIVRST : in std_logic) ;
end component;


-- CoreABC in this example is configured as soft implementation. Therefore the CoreABC program
--instructions are stored in the Embedded Flash Memory of Fusion FPGA 
component Init_Block

    port( INIT_CLK               : in    std_logic;
          SYS_RESET              : in    std_logic;
          INIT_POWER_UP          : in    std_logic;
          INIT_DONE              : out   std_logic;
          INIT_DATA              : out   std_logic_vector(8 downto 0);
          INIT_ADDR              : out   std_logic_vector(11 downto 0);
          COREABC_INITDATVAL : out   std_logic
        );
end component;

component NGMUX
   port(CLK0, CLK1, S : in std_logic; 
        GL : out std_logic) ;
end component;



signal iINITADDR: std_logic_vector (11 downto 0);
signal iINITDATA: std_logic_vector (8 downto 0);
signal iINITDONE, iINITDATVAL: std_logic;
signal CLK40MHZ, CLK10MHZ: std_logic;
signal CLOCK: std_logic;
begin

PLL_inst: PLL_Block port map
    (POWERDOWN => '1',
     CLKA => CLK,
     LOCK => OPEN,
     GLA => CLK10MHZ,
     GLC => CLK40MHZ,
     OADIVRST => RESETn
     );

NGMUX_inst: NGMUX port map
  (CLK0 => CLK10MHZ,
   CLK1 => CLK40MHZ,
   S    => iINITDONE, 
   GL   => CLOCK
   ) ;

InitConfig_inst: Init_Block port map
    ( INIT_CLK        => CLOCK,
      SYS_RESET       => RESETn,       
      INIT_POWER_UP   => '1',
      INIT_DONE       => iINITDONE,       
      INIT_DATA       => iINITDATA,       
      INIT_ADDR       => iINITADDR,       
      COREABC_INITDATVAL => iINITDATVAL
     );

COREABC_AI_inst: CoreABC_AI_Test  port map 
       (AC1 => AC1,
        AT2 => AT2,
        ATRETURN => ATRETURN,
        AV0 => AV0,
        AV1 => AV1,
        INITADDR => iINITADDR,
        INITDATA => iINITDATA,
        INITDATVAL => iINITDATVAL,
        INITDONE => iINITDONE,
        NSYSRESET => RESETn,
        SYSCLK => CLOCK,
        AG3 => AG3,
        AG4 => AG4,
        Flags => Flags,
        VAREF => VAREF
    );

end arch;

